@@ -138,13 +138,14 @@ class SpyderKernel(IPythonKernel):
         if content is None:
             content = {}
         content['spyder_msg_type'] = spyder_msg_type
-        self.session.send(
+        msg = self.session.send(
             self.iopub_socket,
             'spyder_msg',
             content=content,
             buffers=[cloudpickle.dumps(data, protocol=PICKLE_PROTOCOL)],
             parent=self._parent_header,
         )
+        self.log.debug(msg)
 
     def get_value(self, name):
         """Get the value of a variable"""
@@ -178,6 +179,7 @@ class SpyderKernel(IPythonKernel):
         # Deserialize and set value in namespace
         dvalue = cloudpickle.loads(svalue)
         ns[name] = dvalue
+        self.log.debug(ns)
 
     def remove_value(self, name):
         """Remove a variable"""
@@ -503,3 +505,10 @@ class SpyderKernel(IPythonKernel):
         from IPython.core.getipython import get_ipython
         get_ipython().run_line_magic('reload_ext', 'autoreload')
         get_ipython().run_line_magic('autoreload', '2')
+
+    def _load_wurlitzer(self):
+        """Load wurlitzer extension."""
+        # Wurlitzer has no effect on Windows
+        if not os.name == 'nt':
+            from IPython.core.getipython import get_ipython
+            get_ipython().run_line_magic('reload_ext', 'wurlitzer')
