@@ -17,10 +17,8 @@ import io
 import os
 import os.path as osp
 import pdb
-import re
 import shlex
 import sys
-import sysconfig
 import time
 import warnings
 
@@ -320,7 +318,7 @@ class SpyderPdb(pdb.Pdb):
 
     # --- Methods overriden by us
     def preloop(self):
-        """Ask Spyder for berkpoints before the first prompt is created."""
+        """Ask Spyder for breakpoints before the first prompt is created."""
         if self.starting:
             get_ipython().kernel._ask_spyder_for_breakpoints()
 
@@ -541,8 +539,7 @@ class UserModuleReloader(object):
         # pytorch: see issue 7041
         # fastmat: see issue 7190
         # pythoncom: see issue 7190
-        # tensorflow: see issue 8697
-        other_modules = ['pytorch', 'pythoncom', 'tensorflow']
+        other_modules = ['pytorch', 'pythoncom']
         if PY2:
             py2_modules = ['astropy', 'fastmat']
             other_modules = other_modules + py2_modules
@@ -553,20 +550,6 @@ class UserModuleReloader(object):
         self.pathlist = pathlist
         self.previous_modules = list(sys.modules.keys())
 
-    @property
-    def skip_paths(self):
-        """Python library paths to be skipped from module reloading."""
-        try:
-            paths = sysconfig.get_paths()
-            lib_paths = [paths['stdlib'],
-                         paths['purelib'],
-                         paths['scripts'],
-                         paths['data']]
-
-            return lib_paths
-        except Exception:
-            return []
-
     def is_module_blacklisted(self, modname, modpath):
         if HAS_CYTHON:
             # Don't return cached inline compiled .PYX files
@@ -577,37 +560,6 @@ class UserModuleReloader(object):
         else:
             return set(modname.split('.')) & set(self.namelist)
 
-    def is_module_reloadable(self, module):
-        """Decide if a module can be reloaded or not."""
-        modpath = getattr(module, '__file__', None)
-
-        # Skip module according to different criteria
-        if modpath is None:
-            # *module* is a C module that is statically linked into the
-            # interpreter. There is no way to know its path, so we
-            # choose to ignore it.
-            return False
-        elif any([p in modpath for p in self.skip_paths]):
-            # We don't want to reload modules that belong to the
-            # standard library or installed to site-packages,
-            # just modules created by the user.
-            return False
-        elif not os.name == 'nt':
-            # Module paths containing the strings below can be ihherited
-            # from the default Linux installation or Homebrew in a
-            # virtualenv.
-            patterns = [r'^/usr/lib.*',
-                        r'^/usr/local/lib.*',
-                        r'^/usr/.*/dist-packages/.*',
-                        r'^/Library/.*'
-            ]
-            if [p for p in patterns if re.search(p, modpath)]:
-                return False
-            else:
-                return True
-        else:
-            return True
-
     def run(self, verbose=False):
         """
         Del user modules to force Python to deeply reload them
@@ -619,16 +571,15 @@ class UserModuleReloader(object):
         log = []
         for modname, module in list(sys.modules.items()):
             if modname not in self.previous_modules:
-                # Decide if a module can be reloaded or not
-                if not self.is_module_reloadable(module):
+                modpath = getattr(module, '__file__', None)
+                if modpath is None:
+                    # *module* is a C module that is statically linked into the
+                    # interpreter. There is no way to know its path, so we
+                    # choose to ignore it.
                     continue
-
-                # Reload module
                 if not self.is_module_blacklisted(modname, modpath):
                     log.append(modname)
                     del sys.modules[modname]
-
-        # Report reloaded modules
         if verbose and log:
             _print("\x1b[4;33m%s\x1b[24m%s\x1b[0m"\
                    % ("Reloaded modules", ": "+", ".join(log)))
@@ -693,15 +644,29 @@ if "SPYDER_EXCEPTHOOK" in os.environ:
     set_post_mortem()
 
 
-#==============================================================================
+# ==============================================================================
 # runfile and debugfile commands
-#==============================================================================
+# ==============================================================================
 def _get_globals():
     """Return current namespace"""
     ipython_shell = get_ipython()
     return ipython_shell.user_ns
 
 
+def run_umr():
+    """Run the user module reloader."""
+    global __umr__
+    if os.environ.get("SPY_UMR_ENABLED", "").lower() == "true":
+        if __umr__ is None:
+            namelist = os.environ.get("SPY_UMR_NAMELIST", None)
+            if namelist is not None:
+                namelist = namelist.split(',')
+            __umr__ = UserModuleReloader(namelist=namelist)
+        else:
+            verbose = os.environ.get("SPY_UMR_VERBOSE", "").lower() == "true"
+            __umr__.run(verbose=verbose)
+
+
 def runfile(filename, args=None, wdir=None, namespace=None, post_mortem=False):
     """
     Run filename
@@ -715,16 +680,7 @@ def runfile(filename, args=None, wdir=None, namespace=None, post_mortem=False):
         # UnicodeError, TypeError --> eventually raised in Python 2
         # AttributeError --> systematically raised in Python 3
         pass
-    global __umr__
-    if os.environ.get("SPY_UMR_ENABLED", "").lower() == "true":
-        if __umr__ is None:
-            namelist = os.environ.get("SPY_UMR_NAMELIST", None)
-            if namelist is not None:
-                namelist = namelist.split(',')
-            __umr__ = UserModuleReloader(namelist=namelist)
-        else:
-            verbose = os.environ.get("SPY_UMR_VERBOSE", "").lower() == "true"
-            __umr__.run(verbose=verbose)
+    run_umr()
     if args is not None and not isinstance(args, basestring):
         raise TypeError("expected a character buffer object")
     if namespace is None:
@@ -756,9 +712,53 @@ def runfile(filename, args=None, wdir=None, namespace=None, post_mortem=False):
     sys.argv = ['']
     namespace.pop('__file__')
 
+
 builtins.runfile = runfile
 
 
+def runcell(cellname, filename):
+    """
+    Run a code cell from an editor as a file.
+
+    Currently looks for code in an `ipython` property called `cell_code`.
+    This property must be set by the editor prior to calling this function.
+    This function deletes the contents of `cell_code` upon completion.
+
+    Parameters
+    ----------
+    cellname : str
+        Used as a reference in the history log of which
+        cell was run with the fuction. This variable is not used.
+    filename : str
+        Needed to allow for proper traceback links.
+    """
+    try:
+        filename = filename.decode('utf-8')
+    except (UnicodeError, TypeError, AttributeError):
+        # UnicodeError, TypeError --> eventually raised in Python 2
+        # AttributeError --> systematically raised in Python 3
+        pass
+    run_umr()
+    ipython_shell = get_ipython()
+    try:
+        cell_code = ipython_shell.cell_code
+    except AttributeError:
+        _print("--Run Cell Error--\n"
+               "Please use only through Spyder's Editor; "
+               "shouldn't be called manually from the console")
+        return
+
+    # Trigger `post_execute` to exit the additional pre-execution.
+    # See Spyder PR #7310.
+    ipython_shell.events.trigger('post_execute')
+
+    ipython_shell.run_cell(cell_code)
+    del ipython_shell.cell_code
+
+
+builtins.runcell = runcell
+
+
 def debugfile(filename, args=None, wdir=None, post_mortem=False):
     """
     Debug filename
@@ -775,6 +775,7 @@ def debugfile(filename, args=None, wdir=None, post_mortem=False):
         filename = filename.replace('\\', '/')
     debugger.run("runfile(%r, args=%r, wdir=%r)" % (filename, args, wdir))
 
+
 builtins.debugfile = debugfile
 
 
